from lxml.html import document_fromstring
from .wechat import Article, Official
from time import localtime, strftime
from requests import Session, utils
from tempfile import TemporaryFile
from json import loads
from PIL import Image
from re import search

_session = Session()


def search_articles(keyword, pages=1):
    search_urls = [f'http://weixin.sogou.com/weixin?type=2&query={keyword}&page={page}' for page in range(1, pages + 1)]
    articles = []
    for search_url in search_urls:
        html_tree = document_fromstring(_get_html(search_url))
        article_nodes = html_tree.xpath('//*[@class="news-list"]/li')
        for article_node in article_nodes:
            article_url = str(article_node.xpath('./div[2]/h3/a/@href')[0])
            article_title = str(article_node.xpath('./div[2]/h3/a')[0].text_content())
            article_date = strftime('%Y-%m-%d', localtime(int(article_node.xpath('./div[2]/div/@t')[0])))
            article_image_url = str(article_node.xpath('./div[1]/a/img/@src')[0])
            article_digest = str(article_node.xpath('./div[2]/p')[0].text_content())
            official_url = str(article_node.xpath('./div[2]/div/a/@href')[0])
            official_name = str(article_node.xpath('./div[2]/div/a/text()')[0])
            articles.append(
                Article(article_url, article_title, article_date, official_url, official_name, article_digest,
                        article_image_url))
    return articles


def search_officials(keyword, pages=1):
    search_urls = [f'http://weixin.sogou.com/weixin?type=1&query={keyword}&page={page}' for page in range(1, pages + 1)]
    officials = []
    for search_url in search_urls:
        html_text = _get_html(search_url)
        html_tree = document_fromstring(html_text)
        official_nodes = html_tree.xpath('//*[@id="main"]/div[4]/ul/li')
        for official_node in official_nodes:
            officials.append(_parse_official_node(html_text, official_node))
    return officials


def get_official(official_id):
    search_url = f'http://weixin.sogou.com/weixin?type=1&query={official_id}'
    html_text = _get_html(search_url)
    html_tree = document_fromstring(html_text)
    official_node = html_tree.xpath('//*[@id="main"]/div[4]/ul/li[1]')
    if official_node:
        official_node = official_node[0]
        if str(official_id) == str(official_node.xpath('./div/div[2]/p[2]/label/text()')[0]):
            return _parse_official_node(html_text, official_node)
    return None


def _parse_official_node(html_text, official_node):
    official_node_id = str(official_node.xpath('./@d')[0])
    monthly_data_url = 'https://weixin.sogou.com' + search('var account_anti_url = \"(.*?)\";', html_text)[1]
    monthly_data = loads(_get_html(monthly_data_url))['msg']
    official_status = monthly_data[official_node_id].split(',') if official_node_id in monthly_data else ()
    official_id = str(official_node.xpath('./div/div[2]/p[2]/label/text()')[0])
    official_url = str(official_node.xpath('./div/div[2]/p[1]/a/@href')[0])
    official_name = str(official_node.xpath('./div/div[2]/p[1]/a')[0].text_content())
    official_avatar_url = str(official_node.xpath('./div/div[1]/a/img/@src')[0])
    official_qr_code_url = str(official_node.xpath('./div/div[3]/span/img[1]/@src')[0])
    official_profile_desc = str(official_node.xpath('./dl[1]/dd')[0].text_content())
    if official_node.xpath('./dl[2]'):
        recent_article_url = str(official_node.xpath('./dl[2]/dd/a/@href')[0])
        recent_article_title = str(official_node.xpath('./dl[2]/dd/a')[0].text_content())
        recent_article_date = str(official_node.xpath('./dl[2]/dd/span')[0].text_content())
        recent_article_date = int(search('document.write\(timeConvert\(\'(.*?)\'\)\)', recent_article_date)[1])
        recent_article_date = strftime('%Y-%m-%d', localtime(recent_article_date))
        recent_article = Article(recent_article_url, recent_article_title, recent_article_date, official_url,
                                 official_name)
    else:
        recent_article = None
    return Official(official_url, official_id, official_name, official_avatar_url, official_qr_code_url,
                    official_profile_desc, official_status, recent_article)


def _get_html(url):
    resp = _session.get(url)
    if url == resp.url:
        return resp.text
    else:
        cookies = utils.dict_from_cookiejar(_session.cookies)
        snuid = _identify_captcha()
        cookies['SNUID'] = snuid
        cookies['SUV'] = snuid
        _session.cookies = utils.cookiejar_from_dict(cookies)
        return _get_html(url)


def _identify_captcha():
    while True:
        resp = _session.get('http://weixin.sogou.com/antispider/util/seccode.php?tc=100000')
        tf = TemporaryFile()
        tf.write(resp.content)
        captcha_image = Image.open(tf)
        code = _identifying(captcha_image)
        resp = _session.post('https://weixin.sogou.com/antispider/thank.php', data={
            'c': code,
            'r': 'https://weixin.sogou.com/',
            'v': 5
        })
        resp.encoding = 'utf-8'
        msg = resp.json()
        if msg['code'] == 0:
            return msg['id']
        print('验证码输入错误！')


def _identifying(captcha_image):
    captcha_image.show()
    return input('Sougo验证码：')



from trash.official_account import OfficialAccount
from requests import Session, utils
from tempfile import TemporaryFile
from trash.article import Article
from lxml import html
from PIL import Image


class S:
    def __init__(self):
        self._session = Session()

    def search_articles(self, name, pages=1):
        search_urls = [f'http://weixin.sogou.com/weixin?type=2&query={name}&page={page}' for page in
                       range(1, pages + 1)]
        xpath = '//*[@class="news-list"]/li'
        articles = []
        for search_url in search_urls:
            html_tree = self._get_html(search_url)
            article_nodes = html_tree.xpath(xpath)
            for article_node in article_nodes:
                article_url = article_node.xpath('./div[2]/h3/a/@href')[0]
                official_account_url = article_node.xpath('./div[2]/div/a/@href')[0]
                digest = article_node.xpath('./div[2]/p')[0].text_content()
                articles.append(Article(article_url, official_account_url, digest))
        return articles

    def search_official_account(self, wechat_id):
        search_url = f'http://weixin.sogou.com/weixin?type=1&query={wechat_id}'
        html_tree = self._get_html(search_url)
        xpath = '//*[@id="main"]/div[4]/ul/li[1]/div/div[2]'
        official_account = html_tree.xpath(xpath)
        if official_account:
            official_account = official_account[0]
            if wechat_id == official_account.xpath('./p[2]/label/text()')[0]:
                return OfficialAccount(official_account.xpath('./p[1]/a/@href')[0])
        return None

    def _get_html(self, url):
        resp = self._session.get(url)
        if url == resp.url:
            return html.document_fromstring(resp.text)
        else:
            cookies = utils.dict_from_cookiejar(self._session.cookies)
            snuid = self._identify_captcha()
            cookies['SNUID'] = snuid
            cookies['SUV'] = snuid
            self._session.cookies = utils.cookiejar_from_dict(cookies)
            return self._get_html(url)

    def _identify_captcha(self):
        while True:
            resp = self._session.get('http://weixin.sogou.com/antispider/util/seccode.php?tc=100000')
            tf = TemporaryFile()
            tf.write(resp.content)
            image = Image.open(tf)
            code = self._identify(image)
            resp = self._session.post('https://weixin.sogou.com/antispider/thank.php', data={
                'c': code,
                'r': 'https://weixin.sogou.com/',
                'v': 5
            })
            resp.encoding = 'utf-8'
            msg = resp.json()
            if msg['code'] == 0:
                return msg['id']
            print('验证码输入错误！')

    def _identify(self, image):
        image.show()
        return input('Sougo验证码：')



from requests import Session
from lxml import html
from re import search


class Article:

    def __init__(self, url, official_account, digest=None):
        self._url = url
        self._session = Session()
        self._html_tree = self._get_html(url)
        self._title = self._get_title()
        self._official_account = official_account
        self._date = self._get_date()
        self._digest = digest

    def _get_html(self, url):
        return html.document_fromstring(self._session.get(url).text)

    def _get_title(self):
        xpath = '//*[@id="activity-name"]/text()'
        return self._html_tree.xpath(xpath)[0].strip()

    def _get_date(self):
        html_text = html.tostring(self._html_tree)
        return search('var publish_time = "(.*?)" \|\| "";', str(html_text))[1].strip()

    def _get_content(self):
        xpath = '//*[@id="js_content"]'
        return self._html_tree.xpath(xpath)[0]

    def save(self, directory_path):
        with open(f'{directory_path}/{self._title}.md', 'wb') as file:
            content_node = self._get_content()
            contents = [html.tostring(node) for node in content_node]
            text = b'\n'.join(contents)
            text = text.replace(b' data-', b' ')
            file.write(text)

    @property
    def url(self):
        return self._url

    @property
    def title(self):
        return self._title

    @property
    def date(self):
        return self._date

    @property
    def official_account(self):
        return self._official_account

    @property
    def digest(self):
        return self._digest



class A:
    def __init__(self):
        self.b = 0

    def __str__(self):
        return str({'a': 9})

    def __repr__(self):
        return str({'a': 7})

    def __setattr__(self, key, value):
        print(key, value)

# a = A()
# a.b = 6
# A.c = 9
a = [1, 2, 3, 4, 5]
print(a[:3], a[3:])


from os import system, chdir
from shutil import rmtree
from pathlib import Path
from time import sleep

command1 = 'python -m twine upload --repository-url https://upload.pypi.org/legacy/ dist/*'
command2 = 'python3 -m twine upload --repository-url https://test.pypi.org/legacy/ dist/*'
path1 = Path('../build')
path2 = Path('../dist')
path3 = Path('../propor.egg-info')
path_set = [path1, path2, path3]

for p in path_set:
    if p.exists():
        rmtree(p)

chdir('../')
system('python setup.py sdist bdist_wheel')



from time import localtime, strftime
from tempfile import TemporaryFile
from .article import Article
from requests import Session
from re import findall
from json import loads
from lxml import html
from PIL import Image


class OfficialAccount:
    def __init__(self, url):
        self._url = url
        self._session = Session()
        self._html_tree = self._get_html(url)
        self._name = self._get_name()
        self._wechat_id = self._get_wechat_id()
        self._profile_desc = self._get_profile_desc()
        self._account_body = self._get_account_body()
        self._article_items = self._get_article_items()
        self.article_num = len(self._article_items)
        self._articles = self._get_articles()

    def _get_name(self):
        xpath = '/html/body/div/div[1]/div[1]/div[1]/div/strong/text()'
        return self._html_tree.xpath(xpath)[0].strip()

    def _get_wechat_id(self):
        xpath = '/html/body/div/div[1]/div[1]/div[1]/div/p/text()'
        return self._html_tree.xpath(xpath)[0][5:]

    def _get_profile_desc(self):
        xpath = '/html/body/div/div[1]/div[1]/ul/li[1]/div/text()'
        profile_desc = self._html_tree.xpath(xpath)
        return profile_desc[0] if profile_desc else None

    def _get_account_body(self):
        xpath = '/html/body/div/div[1]/div[1]/ul/li[2]/div/text()'
        account_body = self._html_tree.xpath(xpath)
        return account_body[0] if account_body else None

    def _get_article_items(self):
        html_text = html.tostring(self._html_tree, method="html", encoding='utf-8')
        domain_name = 'http://mp.weixin.qq.com'
        result = findall(b'var msgList = {"list":(\[.*?\])};', html_text)
        if not result:
            return []
        article_items = []
        for item in loads(result[0]):
            date = strftime('%Y-%m-%d', localtime(item['comm_msg_info']['datetime']))
            for article_item in item['app_msg_ext_info']['multi_app_msg_item_list']:
                url = article_item['content_url'] if article_item['content_url'].startswith('http') \
                    else domain_name + article_item['content_url']
                article_items.append({
                    'url': url.replace('&amp;', '&'),
                    'title': article_item['title'],
                    'author': article_item['author'],
                    'digest': article_item['digest'],
                    'date': date,
                })
        return article_items

    def _get_articles(self):
        return [Article(item['url'], self, item['digest']) for item in self._article_items]

    @property
    def url(self):
        return self._url

    @property
    def name(self):
        return self._name

    @property
    def wechat_id(self):
        return self._wechat_id

    @property
    def profile_desc(self):
        return self._profile_desc

    @property
    def account_body(self):
        return self._account_body

    @property
    def article_urls(self):
        return [item['url'] for item in self._article_items]

    @property
    def articles(self):
        return self._articles

    @property
    def article_items(self):
        return self._article_items

    def _get_html(self, url):
        resp = self._session.get(url)
        if '请输入验证码' in resp.text:
            self._identify_captcha()
            return self._get_html(url)
        else:
            return html.document_fromstring(resp.text)

    def _identify_captcha(self):
        while True:
            resp = self._session.get('http://mp.weixin.qq.com/mp/verifycode?cert=100000')
            tf = TemporaryFile()
            tf.write(resp.content)
            image = Image.open(tf)
            code = self._identify(image)
            resp = self._session.post('http://mp.weixin.qq.com/mp/verifycode',
                                      data=f'cert={100000}&input={code}&appmsg_token=""')
            resp.encoding = 'utf-8'
            msg = resp.json()
            if msg['ret'] == 0:
                break
            print('验证码输入错误！')

    def _identify(self, image):
        image.show()
        return input('Weixin验证码：')



from requests import Session
from lxml import html

_session = Session()


def search_articles(name, pages=1):
    search_urls = [f'http://weixin.sogou.com/weixin?type=2&query={name}&page={page}' for page in range(1, pages + 1)]
    xpath = '//*[@class="news-list"]/li'
    articles = []
    for search_url in search_urls:
        html_tree = _get_html(search_url)
        article_nodes = html_tree.xpath(xpath)
        for article_node in article_nodes:
            article_url = article_node.xpath('./div[2]/h3/a/@href')[0]
            official_url = article_node.xpath('./div[2]/div/a/@href')[0]
            digest = article_node.xpath('./div[2]/p')[0].text_content()
            articles.append(Article(article_url, official_url, digest))
    return articles


def search_officials(name, pages=1):
    pass


def get_official(id):
    pass


def _get_html(url):
    pass


def _identify_captcha():
    pass


def _identifying(captcha_image):
    pass



from sgwc import search_officials, search_articles, get_official


def test_get_official():
    official = get_official('LOL313298553')
    print(official)
    print(f"""
        url  ---  {official.url}
        official_id  ---  {official.official_id}
        name  ---  {official.name}
        avatar_url  ---  {official.avatar_url}
        qr_code_url  ---  {official.qr_code_url}
        profile_desc  ---  {official.profile_desc}
        recent_article  ---  {official.recent_article}
        articles  ---  {official.articles}
        authenticate  ---  {official.authenticate}
        monthly_articles  ---  {official.monthly_articles}
        monthly_visits  ---  {official.monthly_visits}
    """)


def test_search_articles():
    articles = search_articles('lol')
    print(articles)
    print(f"""
        url  ---  {articles[0].url}
        title  ---  {articles[0].title}
        date  ---  {articles[0].date}
        image_url  ---  {articles[0].image_url}
        digest  ---  {articles[0].digest}
        official_url  ---  {articles[0].official_url}
        official_name  ---  {articles[0].official_name}
        official  ---  {articles[0].official}
    """)
    print(articles[0].official.recent_article, type(articles[0].official.recent_article))


def test_search_officials():
    officials = search_officials('lol')
    print(officials)
    print(officials[0].articles)


def test_get_official_from_url():
    articles = search_articles('lol')
    official = articles[0].official
    print(official)
    print(f"""
        url  ---  {official.url}
        official_id  ---  {official.official_id}
        name  ---  {official.name}
        avatar_url  ---  {official.avatar_url}
        qr_code_url  ---  {official.qr_code_url}
        profile_desc  ---  {official.profile_desc}
        recent_article  ---  {official.recent_article}
        articles  ---  {official.articles}
        authenticate  ---  {official.authenticate}
        monthly_articles  ---  {official.monthly_articles}
        monthly_visits  ---  {official.monthly_visits}
    """)


def test_article_save():
    articles = search_articles('lol')
    for article in articles:
        article.save('markdown')


if __name__ == '__main__':
    # test_get_official()
    # test_search_articles()
    # test_search_officials()
    # test_get_official_from_url()
    test_article_save()



from sgwc import get_official, wechat

# official = get_official('LOL313298553')
# print(official)
# print(official.recent_article)

official = wechat.Official.from_url('http://mp.weixin.qq.com/profile?src=3&timestamp=1550497727&ver=1&signature=i3AiuGb1QwOB5GTeaDJ9C324EzC2B1LrBch1ex6AMI-RrICdniG0z4bU-NfL-A3qx7qJ57qn0KnQgN2AyQ-OnA==')
print(official)
print(official.recent_article)
print(official.authenticate)
print(official.articles)




import setuptools

setuptools.setup(
    name='sgwc',
    version='1.0',
    author='Czw_96',
    author_email='459749926@qq.com',
    description='搜狗微信文章爬虫',
    url='https://github.com/Czw96/sgwc',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'lxml',
        'requests',
        'Pillow',
    ]
)



from lxml.html import document_fromstring, tostring
from tempfile import TemporaryFile
from requests import Session
from .attributes import *
from PIL import Image

_session = Session()


class Article:
    def __str__(self):
        return str(self.info)

    def __repr__(self):
        return str(self.info)

    def __init__(self, url, title, date, official_url, official_name, digest='', image_url=None):
        self._url = url
        self._title = title
        self._date = date
        self._image_url = image_url
        self._digest = digest
        self._official_url = official_url
        self._official_name = official_name
        self._official = None
        self._html_tree = None

    @property
    def url(self):
        return self._url

    @property
    def title(self):
        return self._title

    @property
    def date(self):
        return self._date

    @property
    def image_url(self):
        return self._image_url

    @property
    def digest(self):
        return self._digest

    @property
    def official_url(self):
        return self._official_url

    @property
    def official_name(self):
        return self._official_name

    @property
    def official(self):
        return self._official if self._official else Official.from_url(self._official_url)

    @property
    def info(self):
        return {
            'url': self._url,
            'title': self._title,
            'date': self._date,
            'digest': self._digest,
            'official_url': self._official_url,
            'official_name': self._official_name,
        }

    def save(self, path='.'):
        if not self._html_tree:
            self._html_tree = document_fromstring(_get_html(self._url))
        title = self._title
        title = title.replace('/', '-').replace('\\', '-').replace(':', '：').replace('*', '-')
        title = title.replace('"', '”').replace('|', '-').replace('<', '-').replace('>', '-').replace('?', '？')
        with open(f'{path}/{title}.md', 'w', encoding='utf-8') as file:
            content_node = self._html_tree.xpath('//*[@id="js_content"]')[0]
            contents = [tostring(node, encoding='unicode') for node in content_node]
            text = '\n'.join(contents)
            text = text.replace(' data-', ' ')
            file.write(text)


class Official:
    def __str__(self):
        return str(self.info)

    def __repr__(self):
        return str(self.info)

    def __init__(self, url, official_id, name, avatar_url, qr_code_url, profile_desc, status=(), recent_article=None):
        self._url = url
        self._official_id = official_id
        self._name = name
        self._avatar_url = avatar_url
        self._qr_code_url = qr_code_url
        self._profile_desc = profile_desc
        self._status = status
        self._recent_article = recent_article
        self._articles = []
        self._authenticate = None
        self._html_tree = None

    @property
    def url(self):
        return self._url

    @property
    def official_id(self):
        return self._official_id

    @property
    def name(self):
        return self._name

    @property
    def avatar_url(self):
        return self._avatar_url

    @property
    def qr_code_url(self):
        return self._qr_code_url

    @property
    def profile_desc(self):
        return self._profile_desc

    @property
    def recent_article(self):
        if not self._recent_article:
            self._get_html()
            articles = []
            for article_item in official_articles_(self._html_tree):
                article = Article(**article_item, official_url=self._url, official_name=self._name)
                article._official = self
                articles.append(article)
            self._articles = articles
            self._recent_article = self._articles[0] if self._articles else None
        return self._recent_article

    @property
    def articles(self):
        if not self._articles:
            self._get_html()
            articles = []
            for article_item in official_articles_(self._html_tree):
                article = Article(**article_item, official_url=self._url, official_name=self._name)
                article._official = self
                articles.append(article)
            self._articles = articles
            self._recent_article = self._articles[0] if self._articles else None
        return self._articles

    @property
    def authenticate(self):
        if not self._authenticate:
            self._get_html()
            self._authenticate = official_authenticate_(self._html_tree)
        return self._authenticate

    @property
    def monthly_articles(self):
        return self._status[0] if self._status else None

    @property
    def monthly_visits(self):
        return self._status[1] if self._status else None

    @property
    def info(self):
        return {
            'url': self._url,
            'official_id': self._official_id,
            'name': self._name,
            'profile_desc': self._profile_desc,
            'status': f'月发文 {self._status[0]}篇，月访问 {self._status[1]}次' if self._status else None,
            'recent_article': self._recent_article,
        }

    @classmethod
    def from_url(cls, url):
        html_tree = document_fromstring(_get_html(url))
        official_id = official_id_(html_tree)
        official_name = official_name_(html_tree)
        official_avatar_url = official_avatar_url_(html_tree)
        official_qr_code_url = official_qr_code_url_(html_tree)
        official_profile_desc = official_profile_desc_(html_tree)
        official_authenticate = official_authenticate_(html_tree)
        official_articles = official_articles_(html_tree)
        official = cls(url, official_id, official_name, official_avatar_url, official_qr_code_url,
                       official_profile_desc)
        for article_item in official_articles:
            article = Article(**article_item, official_url=url, official_name=official_name)
            article._official = official
            official._articles.append(article)
        official._authenticate = official_authenticate
        official._recent_article = official._articles[0]
        official._html_tree = html_tree
        return official

    def _get_html(self):
        if self._html_tree is None:
            self._html_tree = document_fromstring(_get_html(self._url))


def _get_html(url):
    resp = _session.get(url)
    if '请输入验证码' in resp.text:
        _identify_captcha()
        return _get_html(url)
    else:
        return resp.text


def _identify_captcha():
    while True:
        resp = _session.get('http://mp.weixin.qq.com/mp/verifycode?cert=100000')
        tf = TemporaryFile()
        tf.write(resp.content)
        captcha_image = Image.open(tf)
        code = _identifying(captcha_image)
        resp = _session.post('http://mp.weixin.qq.com/mp/verifycode',
                             data=f'cert={100000}&input={code}&appmsg_token=""')
        resp.encoding = 'utf-8'
        msg = resp.json()
        if msg['ret'] == 0:
            break
        print('验证码输入错误！')


def _identifying(captcha_image):
    captcha_image.show()
    return input('WeChat验证码：')



from time import strftime, localtime
from lxml.html import tostring
from re import findall
from json import loads

__all__ = ['official_id_', 'official_name_', 'official_avatar_url_', 'official_qr_code_url_', 'official_profile_desc_',
           'official_authenticate_', 'official_articles_']


def official_id_(html_tree):
    return html_tree.xpath('/html/body/div/div[1]/div[1]/div[1]/div/p/text()')[0][5:]


def official_name_(html_tree):
    return html_tree.xpath('/html/body/div/div[1]/div[1]/div[1]/div/strong/text()')[0].strip()


def official_avatar_url_(html_tree):
    return html_tree.xpath('/html/body/div/div[1]/div[1]/div[1]/span/img/@src')[0]


def official_qr_code_url_(html_tree):
    return html_tree.xpath('//*[@id="js_pc_qr_code_img"]/@src')[0]


def official_profile_desc_(html_tree):
    profile_desc = html_tree.xpath('/html/body/div/div[1]/div[1]/ul/li[1]/div/text()')
    return profile_desc[0] if profile_desc else None


def official_authenticate_(html_tree):
    authenticate = html_tree.xpath('/html/body/div/div[1]/div[1]/ul/li[2]/div/text()')
    return authenticate[0] if authenticate else None


def official_articles_(html_tree):
    html_text = tostring(doc=html_tree, encoding='unicode')
    domain_name = 'http://mp.weixin.qq.com'
    result = findall('var msgList = {"list":(\[.*?\])};', html_text)
    if not result:
        return []
    article_items = []
    for item in loads(result[0]):
        date = strftime('%Y-%m-%d', localtime(item['comm_msg_info']['datetime']))
        for article_item in item['app_msg_ext_info']['multi_app_msg_item_list']:
            url = article_item['content_url'] if article_item['content_url'].startswith('http') \
                else domain_name + article_item['content_url']
            article_items.append({
                'url': url.replace('&amp;', '&'),
                'title': article_item['title'],
                'date': date,
                'digest': article_item['digest'],
                'image_url': article_item['cover'],
            })
    return article_items
